from django.apps import AppConfig

from django.utils.translation import ugettext_lazy as _


class WebDesignConfig(AppConfig):
    name = 'django.contrib.webdesign'
    verbose_name = _("Web Design")
